{=========================================================================
  RkBwcc Unit

  The components in this unit (TRkBwccCheckBox and TRkBwccRadioButton)
  are component wrappers for the corresponding Borland Style controls
  found in the BWCC custom control library (i.e. BWCC32.DLL).

  Developing Custom Delphi 3 Components
  Copyright  1995-1997 by Ray Konopka
=========================================================================}

{$I DCDC.INC}

unit RkBwcc;

interface

uses
  Messages, Classes, Graphics, Controls, StdCtrls, Menus, RkCommon;

type
  TRkBwccCheckBox = class( TWinControl )
  private
    FAboutInfo : TRkAboutInfo;
    FAlignment : TLeftRight;
    FAllowGrayed : Boolean;
    FState : TCheckBoxState;
    procedure SetAlignment( Value : TLeftRight );
    function GetChecked: Boolean;
    procedure SetChecked( Value : Boolean );
    procedure SetState( Value : TCheckBoxState );
    procedure CMDialogChar(var Msg: TCMDialogChar); message cm_DialogChar;
    procedure CNCommand( var Msg : TWMCommand ); message cn_Command;
  protected
    procedure ToggleState; virtual;
    procedure CreateParams( var Params : TCreateParams ); override;
    procedure CreateWnd; override;
  public
    constructor Create( AOwner : TComponent ); override;
  published
    property About : TRkAboutInfo
      read FAboutInfo
      write FAboutInfo
      stored False;

    property Alignment: TLeftRight
      read FAlignment
      write SetAlignment
      default taRightJustify;

    property AllowGrayed: Boolean
      read FAllowGrayed
      write FAllowGrayed
      default False;

    property Checked: Boolean
      read GetChecked
      write SetChecked
      stored False;

    property State: TCheckBoxState
      read FState
      write SetState
      default cbUnchecked;


    { Inherited Properties and Events }
    property Caption;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;


  TRkBwccRadioButton = class( TButtonControl )
  private
    FAboutInfo : TRkAboutInfo;
    FAlignment : TLeftRight;
    FChecked : Boolean;
    procedure SetAlignment( Value : TLeftRight );
    procedure SetChecked( Value : Boolean );
    procedure CMDialogChar(var Msg: TCMDialogChar); message cm_DialogChar;
    procedure CNCommand( var Msg : TWMCommand ); message cn_Command;
  protected
    procedure CreateParams( var Params : TCreateParams ); override;
    procedure CreateWnd; override;
  public
    constructor Create( AOwner : TComponent ); override;
  published
    property About : TRkAboutInfo
      read FAboutInfo
      write FAboutInfo
      stored False;

    property Alignment : TLeftRight
      read FAlignment
      write SetAlignment
      default taRightJustify;

    property Checked : Boolean
      read FChecked
      write SetChecked
      default False;

    { Inherited Properties and Events }
    property Caption;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;


implementation

uses
  Windows, SysUtils, Forms;

const
  BwccLib = 'BWCC32.DLL';
  SLoadLibraryError = 'Failed to load %s for %s. '#13 +
                      'LoadLibrary error code %d.';

var
  BwccHandle : THandle = 0;        // Handle for BWCC Dynamic Link Library


{=============================}
{== TRkBwccCheckBox Methods ==}
{=============================}

constructor TRkBwccCheckBox.Create( AOwner : TComponent );
begin
  if BwccHandle < HINSTANCE_ERROR then
    raise EComponentError.CreateFmt( SLoadLibraryError,
                                     [ BwccLib, 'TRkBwccCheckBox',
                                       BwccHandle ] );
  inherited Create( AOwner );
  ControlStyle := [ csSetCaption, csDoubleClicks ];
  FAlignment := taRightJustify;    // Default Caption on right side of box
  Height := 17;
  Width := 140;
  TabStop := True;
  FState := cbUnchecked;
end;


procedure TRkBwccCheckBox.CreateParams( var Params : TCreateParams );
begin
  inherited CreateParams( Params );

  // Specify the BWCC class name for check box control
  CreateSubClass( Params, 'BorCheck' );

  with Params do
  begin
    // Use the bs_3State style and not bs_Auto3State, so that we can
    // control the display of the Grayed state using the AllowGrayed
    // property.

    Style := Style or bs_3State;
    if FAlignment = taLeftJustify then
      Style := Style or bs_LeftText;     // Add style flag if left aligned
  end;
end;


procedure TRkBwccCheckBox.CreateWnd;
begin
  inherited CreateWnd;
  SendMessage( Handle, bm_SetCheck, Integer( FState ), 0 );
end;


procedure TRkBwccCheckBox.ToggleState;
begin
  // Cycle through states:  [Grayed ->] Checked -> Unchecked
  case FState of
    cbUnchecked:
    begin
      if AllowGrayed then
        State := cbGrayed
      else
        State := cbChecked;
    end;

    cbChecked:
      State := cbUnchecked;

    cbGrayed:
      State := cbChecked;
  end;
end;


procedure TRkBwccCheckBox.SetAlignment( Value : TLeftRight );
begin
  if Value <> FAlignment then
  begin
    FAlignment := Value;
    RecreateWnd;          // Recreate the window handle with the new style
  end;
end;


function TRkBwccCheckBox.GetChecked : Boolean;
begin
  Result := SendMessage( Handle, bm_GetCheck, 0, 0 ) = Ord( cbChecked );
end;


procedure TRkBwccCheckBox.SetChecked( Value : Boolean );
begin
  if Value then
    State := cbChecked
  else
    State := cbUnchecked;
end;


procedure TRkBwccCheckBox.SetState( Value : TCheckBoxState );
begin
  if Value <> State then
  begin
    FState := Value;
    if HandleAllocated then
      SendMessage( Handle, bm_SetCheck, Integer( FState ), 0 );

    Click;                                    // Trigger the OnClick event
  end;
end;


{=========================================================================
  TRkBwccCheckBox.CMDialogChar

  The cmDialogChar component message is handle so that if the user
  presses the Access key for the control (e.g. Alt+C), the component will
  gain the focus and toggle its state.
=========================================================================}

procedure TRkBwccCheckBox.CMDialogChar( var Msg : TCMDialogChar );
begin
  with Msg do
  begin
    if IsAccel( CharCode, Caption ) and CanFocus then
    begin
      SetFocus;
      if Focused then
        ToggleState;          // If able to change focus, toggle its state
      Result := 1;    // Indicate to Windows that message has been handled
    end
    else
      inherited;
  end;
end;


{=========================================================================
  TRkBwccCheckBox.CNCommand

  This component notification message is used to indicate when the user
  clicked on the component.  This method is responsible for changing the
  state of the check box.
=========================================================================}

procedure TRkBwccCheckBox.CNCommand( var Msg : TWMCommand );
begin
  if Msg.NotifyCode = bn_Clicked then
    ToggleState;
end;



{================================}
{== TRkBwccRadioButton Methods ==}
{================================}

constructor TRkBwccRadioButton.Create( AOwner : TComponent );
begin
  if BwccHandle < HINSTANCE_ERROR then
    raise EComponentError.CreateFmt( SLoadLibraryError,
                                     [ BwccLib, 'TRkBwccRadioButton',
                                       BwccHandle ] );
  inherited Create( AOwner );
  ControlStyle := [ csSetCaption, csDoubleClicks ];
  FAlignment := taRightJustify;           // Default Caption on right side
  Height := 17;
  Width := 140;
  TabStop := True;
end;


procedure TRkBwccRadioButton.CreateParams( var Params : TCreateParams );
begin
  inherited CreateParams( Params );

  // Specify the BWCC class name for radio button control
  CreateSubClass( Params, 'BorRadio' );

  with Params do
  begin
    Style := Style or bs_AutoRadioButton;
    if FAlignment = taLeftJustify then
      Style := Style or bs_LeftText;     // Add style flag if left aligned
  end;
end;


procedure TRkBwccRadioButton.CreateWnd;
begin
  inherited CreateWnd;
  SendMessage( Handle, bm_SetCheck, Integer( FChecked ), 0 );
end;


procedure TRkBwccRadioButton.SetAlignment( Value : TLeftRight );
begin
  if Value <> FAlignment then
  begin
    FAlignment := Value;
    RecreateWnd;              // Recreate the window handle with new style
  end;
end;


procedure TRkBwccRadioButton.SetChecked( Value : Boolean );

  procedure TurnOffSiblings;
  var
    I : Integer;
    Sibling : TControl;
  begin
    if Parent <> nil then
    begin
      for I := 0 to Parent.ControlCount - 1 do
      begin
        Sibling := Parent.Controls[ I ];
        if ( Sibling <> Self ) and ( Sibling is TRkBwccRadioButton ) then
          TRkBwccRadioButton( Sibling ).SetChecked( False );
      end;
    end;
  end; {= TurnOffSiblings =}

begin {= TRkBwccRadioButton.SetChecked =}
  if Value <> FChecked then
  begin
    FChecked := Value;
    TabStop := Value;
    if HandleAllocated then
      SendMessage( Handle, bm_SetCheck, Integer( FChecked ), 0 );

    if FChecked then
    begin
      TurnOffSiblings;
      Click;                                      // Trigger OnClick event
    end;
  end;
end; {= TRkBwccRadioButton.SetChecked =}


procedure TRkBwccRadioButton.CMDialogChar( var Msg : TCMDialogChar );
begin
  with Msg do
  begin
    if IsAccel( CharCode, Caption ) and CanFocus then
    begin
      SetFocus;
      Result := 1;    // Indicate to Windows that message has been handled
    end
    else
      inherited;
  end;
end;


procedure TRkBwccRadioButton.CNCommand( var Msg : TWMCommand );
begin
  case Msg.NotifyCode of
    bn_Clicked:
      SetChecked( True );

    bn_DoubleClicked :
      DblClick;
  end;
end;


// Variables needed during initialization
var
  BWCCRegister : function( Instance : THandle ) : Bool; stdcall;
  ErrMode : Integer;


initialization
  // Prevent Windows error dialog from displaying
  ErrMode := SetErrorMode( sem_NoOpenFileErrorBox );
  BwccHandle := LoadLibrary( BwccLib );
  SetErrorMode( ErrMode );

  if BwccHandle >= HINSTANCE_ERROR then
  begin                    // Get the address of the BWCCRegister function
    @BWCCRegister := GetProcAddress( BwccHandle, 'BWCCRegister' );
    {$IFDEF VER90}
    BWCCRegister( HInstance );       // Register the application with BWCC
    {$ELSE}
    BWCCRegister( MainInstance );
    {$ENDIF}
  end;

finalization
  if BwccHandle >= HINSTANCE_ERROR then  // If DLL was successfully loaded
    FreeLibrary( BwccHandle );                                  // Free it

end.

