{=========================================================================
  RkDBStat Unit

  This unit defines the TRkDBStatusControl class, which is a data-aware
  version of the TRkStatusControl component.

  Developing Custom Delphi 3 Components
  Copyright  1995-1997 by Ray Konopka
=========================================================================}

{$I DCDC.INC}

unit RkDbStat;

interface

uses
  Messages, WinTypes, WinProcs, Classes, Graphics, Controls,
  {$IFDEF VER100}
  DBCtrls,
  {$ELSE}
  DBTables,
  {$ENDIF}
  DB, RkStatus, RkCommon;

type
  TRkDBStatusControl = class( TRkCustomStatusControl )
  private
    FDataLink : TFieldDataLink;
    FFieldLabel : string;

    function GetDataField : string;
    procedure SetDataField( const Value : string );
    function GetDataSource : TDataSource;
    procedure SetDataSource( Value : TDataSource );

    procedure SetFieldLabel( const Value : string );
    procedure DataChange( Sender : TObject );
    procedure CMGetDataLink( var Msg : TMessage ); message cm_GetDataLink;
  protected
    procedure Notification( AComponent : TComponent;
                            Operation : TOperation ); override;
  public
    constructor Create( AOwner : TComponent ); override;
    destructor Destroy; override;
  published
    property DataField : string
      read GetDataField
      write SetDataField;

    property DataSource : TDataSource
      read GetDataSource
      write SetDataSource;

    property FieldLabel : string
      read FFieldLabel
      write SetFieldLabel;

    { Inherited Properties }
    property Alignment;
    property CaptionOffset;

    // No Need to surface Caption property b/c the DataSource will supply
    // the display string
  end; 


implementation

{================================}
{== TRkDBStatusControl Methods ==}
{================================}

constructor TRkDBStatusControl.Create( AOwner : TComponent );
begin
  inherited Create( AOwner );

  // Must add csReplicatable in order to use control in a DBCtrlGrid
  ControlStyle := ControlStyle + [ csReplicatable ];

  FFieldLabel := '';

  FDataLink := TFieldDataLink.Create;                   // Create DataLink

  FDataLink.OnDataChange := DataChange;            // Assign event handler
end;


destructor TRkDBStatusControl.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;


procedure TRkDBStatusControl.Notification( AComponent : TComponent;
                                           Operation : TOperation );
begin
  inherited Notification( AComponent, Operation );
  if ( Operation = opRemove ) and
     ( FDataLink <> nil ) and
     ( AComponent = DataSource ) then
    DataSource := nil;
end;


function TRkDBStatusControl.GetDataField : string;
begin
  Result := FDataLink.FieldName;
end;


procedure TRkDBStatusControl.SetDataField( const Value : string );
begin
  FDataLink.FieldName := Value;
end;


function TRkDBStatusControl.GetDataSource : TDataSource;
begin
  Result := FDataLink.DataSource;
end;


procedure TRkDBStatusControl.SetDataSource( Value : TDataSource );
begin
  if FDatalink.DataSource <> Value then
  begin
    FDataLink.DataSource := Value;

    if Value <> nil then
      Value.FreeNotification( Self );
  end;
end;


{=========================================================================
  TRkDBStatusControl.DataChange

  This method gets called as a result of a number of different events:

  1. The underlying field value changes.  Occurs when changing the value
     of the column tied to this control and then move to a new column or
     a new record.
  2. The corresponding Dataset goes into Edit mode.
  3. The corresponding Dataset referenced by DataSource changes.
  4. The current cursor is scrolled to a new record in the table.
  5. The record is reset through a Cancel call.
  6. The DataField property changes to reference another column.
=========================================================================}

procedure TRkDBStatusControl.DataChange( Sender : TObject );
begin
  if FDataLink.Field = nil then
    Caption := ''
  else
    Caption := FFieldLabel + FDataLink.Field.DisplayText;
end;


procedure TRkDBStatusControl.SetFieldLabel( const Value : string );
begin
  if Value <> FFieldLabel then
  begin
    FFieldLabel := Value;
    DataChange( Self );                     // Repaint with new FieldLabel
  end;
end;


procedure TRkDBStatusControl.CMGetDataLink( var Msg : TMessage );
begin
  // Must handle this message if component is to be used in a DBCtrlGrid
  Msg.Result := Integer( FDataLink );
end;


end.
