{=========================================================================
  RkLookFm Unit                                                             
                                                                            
  This is the form unit for the TRkFrmLookup form. This form is displayed
  by the TRkLookupDialog component when its Execute method is called.
  This dialog box form contains an edit field and a database grid. The
  edit field is used to enter keyboard searches on a particular column.

  Developing Custom Delphi 3 Components
  Copyright  1995-1997 by Ray Konopka
=========================================================================}

{$I DCDC.INC}

unit RkLookFm;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Grids, DBGrids, DB, Buttons, ExtCtrls;

type
  TRkLookupForm = class(TForm)
    SrcLookup: TDataSource;
    PnlPrompt: TPanel;
    PnlLookup: TPanel;
    PnlButtons: TPanel;
    PnlButtonOffset: TPanel;
    GrdLookup: TDBGrid;
    EdtSearch: TEdit;
    LblPrompt: TLabel;
    BtnOK: TButton;
    BtnCancel: TButton;
    procedure EdtSearchChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure EdtSearchKeyPress(Sender: TObject; var Key: Char);
  private
  public
    SearchField : string;
    NumbersOnly : Boolean;
  end;

var
  RkLookupForm: TRkLookupForm;

implementation

{$R *.DFM}

uses
  DBTables;


{===========================}
{== TRkLookupForm Methods ==}
{===========================}

{=========================================================================
  TRkLookupForm.EdtSearchChange

  This method is called whenever the contents of the EdtSearch edit field
  changes.  As the user enters characters, this method uses the Locate
  method to find a match.
=========================================================================}

procedure TRkLookupForm.EdtSearchChange(Sender: TObject);
begin

  with SrcLookup.Dataset do
  begin
    DisableControls;          // Disable controls so grid does not flicker
    try
      if EdtSearch.Text = '' then
        First
      else
        Locate( SearchField, EdtSearch.Text,
                [ loPartialKey, loCaseInsensitive ] );
    finally
      EnableControls;                    // Be sure to enable the controls
    end;
  end; { with }
end;


procedure TRkLookupForm.FormShow(Sender: TObject);
begin
  EdtSearch.Width := GrdLookup.Width;             // Resize the edit field
  EdtSearch.SetFocus;             // Reset the focus if dialog redisplayed
  SrcLookup.Dataset.First;          // Move cursor to first record in grid
end;


procedure TRkLookupForm.FormResize(Sender: TObject);
begin
  EdtSearch.Width := GrdLookup.Width;             // Resize the edit field
end;


procedure TRkLookupForm.EdtSearchKeyPress(Sender: TObject; var Key: Char);
begin
  if NumbersOnly and
     ( Key in [ #32..#255 ] ) and
     not ( Key in [ '-', '+', '0'..'9' ] ) then
  begin
    MessageBeep( 0 );
    Key := #0;
  end;
end;

end.
