{=========================================================================
  RzDebug Unit

  This unit defines a Text File Device Driver that defines an interface
  between the Object Pascal file system and the RzDBMsgs OLE Server.
  This allows any Delphi application to send messages to the RzDBMsgs
  program using the standard Write and Writeln procedures.

  Copyright  1995-1997 by Raize Software Solutions, Inc.
=========================================================================}

unit RzDebug;

interface

uses
  Windows;

procedure AssignDbg( var F : Text );
procedure DbgInsertCheckPoint;
procedure WritePoint( Prompt : string; P : TPoint );
procedure WriteRect( Prompt : string; R : TRect );

implementation

uses
  SysUtils, Forms,
  {$IFDEF VER100} ComObj, {$ELSE} OleAuto, {$ENDIF}
  Dialogs;

var
  DbgMsgs : Variant;                 // Variant to hold link to Ole Server


{================================}
{== Device Interface Functions ==}
{================================}

function DbgClose( var F : TTextRec ) : Integer; far;
begin
  Result := 0;
end;


function DbgOutput( var F : TTextRec ) : Integer; far;
var
  Success : Boolean;
begin
  if F.BufPos <> 0 then
  begin
    F.Buffer[ F.BufPos ] := #0;
    repeat
      Success := True;
      try
        DbgMsgs.LogMessage( '['+Application.Title+'] ' + F.BufPtr );
      except
        on EOleError do
        begin
          Success := False;
          DbgMsgs := CreateOleObject( 'RzDbMsgs.DebugMsgsApp' );
        end;
      end;
    until Success;
    F.BufPos := 0;
  end;
  Result := 0;
end;


function DbgOpen( var F : TTextRec ) : Integer; far;
begin
  F.Mode := fmOutput;                        // Always Open in Output Mode
  F.InOutFunc := @DbgOutput;
  F.FlushFunc := @DbgOutput;
  F.CloseFunc := @DbgClose;
  Result := 0;
end;


{=============================}
{== Assign Device Procedure ==}
{=============================}

procedure AssignDbg( var F : Text );
begin
  with TTextRec( F ) do
  begin
    Handle := $FFFF;
    Mode := fmClosed;
    BufSize := SizeOf( Buffer );
    BufPtr := @Buffer;
    OpenFunc := @DbgOpen;
    Name[ 0 ] := #0;
  end;
end;


procedure DbgInsertCheckPoint;
var
  Success : Boolean;
begin
  repeat
    Success := True;
    try
      DbgMsgs.InsertCheckPoint;
    except
      on EOleError do
      begin
        Success := False;
        DbgMsgs := CreateOleObject( 'RzDbMsgs.DebugMsgsApp' );
      end;
    end;
  until Success;
end;


procedure WritePoint( Prompt : string; P : TPoint );
begin
  with P do
    Writeln( Prompt + ': ( ', X, ', ', Y, ' )' );
end;

procedure WriteRect( Prompt : string; R : TRect );
begin
  with R do
    Writeln( Prompt + ': ( ', Left, ', ', Top, ', ',
             Right, ', ', Bottom, ' )' );
end;


initialization
  DbgMsgs := CreateOleObject( 'RzDbMsgs.DebugMsgsApp' );
  AssignDbg( Output );
  Rewrite( Output );
end.
